@@ -2,9 +2,6 @@ package Command::V2;  # additional methods to dispatch from a command-line
 use strict;
 use warnings;
 
-our $entry_point_class;
-our $entry_point_bin;
-
 sub execute_with_shell_params_and_exit {
     # This automatically parses command-line options and "does the right thing":
     # TODO: abstract out all dispatchers for commands into a given API
@@ -14,10 +11,10 @@ sub execute_with_shell_params_and_exit {
         die "No params expected for execute_with_shell_params_and_exit()!";
     }
 
-    $entry_point_class ||= $class;
-    $entry_point_bin ||= File::Basename::basename($0);
+    $Command::entry_point_class ||= $class;
+    $Command::entry_point_bin ||= File::Basename::basename($0);
 
-    if ($ENV{COMP_LINE}) {
+    if ($ENV{COMP_CWORD}) {
         require Getopt::Complete;
         my @spec = $class->resolve_option_completion_spec();
         my $options = Getopt::Complete::Options->new(@spec);
@@ -44,12 +41,14 @@ sub _execute_with_shell_params_and_return_exit_code {
     my $class = shift;
     my @argv = @_;
 
+    my $original_cmdline = join("\0",$0,@argv);
+
     # make --foo=bar equivalent to --foo bar
     @argv = map { ($_ =~ /^(--\w+?)\=(.*)/) ? ($1,$2) : ($_) } @argv;
     my ($delegate_class, $params, $errors) = $class->resolve_class_and_params_for_argv(@argv);
 
     my $exit_code;
-    if ($errors and @$errors and not $params) {
+    if ($errors and @$errors) {
         $delegate_class->dump_status_messages(1);
         $delegate_class->dump_warning_messages(1);
         $delegate_class->dump_error_messages(1);
@@ -59,7 +58,7 @@ sub _execute_with_shell_params_and_return_exit_code {
         $exit_code = 1;
     }
     else {
-        my $rv = $class->_execute_delegate_class_with_params($delegate_class,$params);
+        my $rv = $class->_execute_delegate_class_with_params($delegate_class,$params,$original_cmdline);
         $exit_code = $delegate_class->exit_code_for_return_value($rv);
     }
 
@@ -69,12 +68,13 @@ sub _execute_with_shell_params_and_return_exit_code {
 
 sub _execute_delegate_class_with_params {
     # this is called by both the shell dispatcher and http dispatcher for now
-    my ($class, $delegate_class, $params) = @_;
+    my ($class, $delegate_class, $params, $original_cmdline) = @_;
 
     unless ($delegate_class) {
         $class->dump_status_messages(1);
         $class->dump_warning_messages(1);
         $class->dump_error_messages(1);
+        $class->dump_usage_messages(1);
         $class->dump_debug_messages(0);
         $class->usage_message($class->help_usage_complete_text);
         return;
@@ -83,8 +83,13 @@ sub _execute_delegate_class_with_params {
     $delegate_class->dump_status_messages(1);
     $delegate_class->dump_warning_messages(1);
     $delegate_class->dump_error_messages(1);
+    $delegate_class->dump_usage_messages(1);
     $delegate_class->dump_debug_messages(0);
 
+    # FIXME There should be a better check for params that are there because they came from the
+    # command line, and params that exist for infrastructural purposes.  'original_command_line'
+    # won't ever be given on the command line and shouldn't count toward the next test.
+    # maybe check the is_input properties...
     if ( !defined($params) ) {
         my $command_name = $delegate_class->command_name;
         $delegate_class->status_message($delegate_class->help_usage_complete_text);
@@ -97,6 +102,7 @@ sub _execute_delegate_class_with_params {
         return 1;
     }
 
+    $params->{'original_command_line'} = $original_cmdline if (defined $original_cmdline);
     my $command_object = $delegate_class->create(%$params);
 
     unless ($command_object) {
@@ -148,6 +154,12 @@ sub resolve_class_and_params_for_argv {
         my @errors;
         local $SIG{__WARN__} = sub { push @errors, @_ };
         
+        ## Change the pattern to be '--', '-' followed by a non-digit, or '+'.
+        ## This s the effect of treating a negative number as a value of an option.
+        ## This means that we won't be allowed to have an option named, say, -1.
+        ## But since command modules' properties have to be allowable function names,
+        ## and "1" is not a valid function name, it's not really a problem
+        #Getopt::Long::Configure('prefix_pattern=--|-(?!\D)|\+');
         unless (GetOptions($params_hash,@spec)) {
             for my $error (@errors) {
                 $self->error_message($error);
@@ -169,21 +181,18 @@ sub resolve_class_and_params_for_argv {
             }
             my $value = $ARGV[$n];
             my $meta = $self->__meta__->property_meta_for_name($name);
-            if ($meta->is_many) {
-                if ($n == $#names) {
-                    # slurp the rest
-                    $params_hash->{$name} = [@ARGV[$n..$#ARGV]];
-                    last;
-                }
-                else {
-                    die "has-many property $name is not last in bare_shell_argument_names for $self?!";
-                }
+            if ($meta->is_many and $n == $#names) {
+                # slurp the rest
+                $params_hash->{$name} = [@ARGV[$n..$#ARGV]];
+                last;
             }
             else {
                 $params_hash->{$name} = $value;
             }
         }
-    } elsif (@ARGV) {
+    } 
+    
+    if (@ARGV and not $self->_bare_shell_argument_names) {
         ## argv but no names
         $self->error_message("Unexpected bare arguments: @ARGV!");
         return($self, undef);
@@ -199,6 +208,10 @@ sub resolve_class_and_params_for_argv {
                 push @new_value, @parts;
             }
             @$value = @new_value;
+
+        } elsif ($value eq q('') or $value eq q("")) {
+            # Handle the special values '' and "" to mean undef/NULL
+            $params_hash->{$key} = '';
         }
 
         # turn dashes into underscores
@@ -215,14 +228,29 @@ sub resolve_class_and_params_for_argv {
         $params_hash->{$new_key} = delete $params_hash->{$key};
     }
 
+    # futher work is looking for errors, and may display them
+    # if help is set, return now
+    # we might have returned sooner, but having full info available
+    # allows for dynamic help
+    if ($params_hash->{help}) {
+        return ($self, $params_hash);
+    }
+
     ##
     my $params = $params_hash;
     my $class = $self->class;
+    
+    if (my @errors = $self->_errors_from_missing_parameters($params)) {
+        return ($class, $params, \@errors);
+    }
 
-    unless (@_ && scalar($self->_missing_parameters($params)) == 0) {
+    unless (@_) {
         return ($class, $params);
     }
 
+    # should this be moved up into the methods which are only called
+    # directly from the shell, or is it okay everywhere in this module to
+    # presume we're a direct cmdline call? -ssmith
     local $ENV{UR_COMMAND_DUMP_STATUS_MESSAGES} = 1;
 
     my @params_to_resolve = $self->_params_to_resolve($params);
@@ -256,7 +284,6 @@ sub resolve_class_and_params_for_argv {
                 properties => [$p->{name}],
                 desc => "Problem resolving from $param_arg_str.",
             );
-            $self->error_message();
         }
     }
 
@@ -278,7 +305,7 @@ sub resolve_option_completion_spec {
     return \@completion_spec
 }
 
-sub _missing_parameters {
+sub _errors_from_missing_parameters {
     my ($self, $params) = @_;
 
     my $class_meta = $self->__meta__;
@@ -289,7 +316,7 @@ sub _missing_parameters {
     }
     my @property_metas = map { $class_meta->property_meta_for_name($_); } @property_names;
 
-    my @missing_property_values;
+    my @error_tags;
     for my $property_meta (@property_metas) {
         my $pn = $property_meta->property_name;
 
@@ -298,15 +325,35 @@ sub _missing_parameters {
         next if defined $property_meta->default_value;
         next if defined $params->{$pn};
 
-        push @missing_property_values, $pn;
+        my $arg = $pn;
+        $arg =~ s/_/-/g;
+        $arg = "--$arg";
+
+        if ($property_meta->is_output and not $property_meta->is_input and not $property_meta->is_param) {
+            if ($property_meta->_data_type_as_class_name->__meta__->data_source) {
+                # outputs with a data source do not need a specification 
+                # on the cmdline to "store" them after execution
+                next;
+            }
+            else {
+                push @error_tags, UR::Object::Tag->create(
+                    type => 'invalid',
+                    properties => [$pn],
+                    desc => "Output requires specified destination: " . $arg . "."
+                );
+            }
+        }
+        else {
+            $DB::single = 1;
+            push @error_tags, UR::Object::Tag->create(
+                type => 'invalid',
+                properties => [$pn],
+                desc => "Missing required parameter: " . $arg . "."
+            );
+        }
     }
 
-    @missing_property_values = map { $_ =~ s/_/-/g; "--$_" } @missing_property_values;
-    if (@missing_property_values) {
-        $self->status_message('');
-        $self->error_message("Missing required parameter(s): " . join(', ', @missing_property_values) . ".");
-    }
-    return @missing_property_values;
+    return @error_tags;
 }
 
 sub _params_to_resolve {
@@ -399,15 +446,27 @@ sub _shell_args_property_meta {
     my $rule = UR::Object::Property->define_boolexpr(@_);
     my %seen;
     my (@positional,@required,@optional);
-    foreach my $property_meta ( $class_meta->get_all_property_metas() ) {
+
+    my @properties_with_position = map { [ $_->position_in_module_header, $_ ] }
+                                   $class_meta->properties();
+    my @sorted =
+        sort { 
+            $a->[0] <=> $b->[0]
+        } 
+        @properties_with_position;
+    my @property_meta = map { $_->[1] } @sorted;
+    foreach my $property_meta (@property_meta) {
         my $property_name = $property_meta->property_name;
 
         next if $seen{$property_name}++;
         next unless $rule->evaluate($property_meta);
 
+        next unless $property_meta->can("is_param") and ($property_meta->is_param or $property_meta->is_input);
+
         next if $property_name eq 'id';
         next if $property_name eq 'result';
         next if $property_name eq 'is_executed';
+        next if $property_name eq 'original_command_line';
         next if $property_name =~ /^_/;
 
         next if $property_meta->implied_by;
@@ -434,14 +493,18 @@ sub _shell_args_property_meta {
         }
     }
 
+    @required   = map { [ $_->position_in_module_header, $_ ] } @required;
+    @optional   = map { [ $_->position_in_module_header, $_ ] } @optional;
+    @positional = map { [ $_->{shell_args_position}, $_ ] } @positional;
+
     my @result;
     @result = ( 
-        (sort { $a->property_name cmp $b->property_name } @required),
-        (sort { $a->property_name cmp $b->property_name } @optional),
-        (sort { $a->{shell_args_position} <=> $b->{shell_args_position} } @positional),
+        (sort { $a->[0] cmp $b->[0] } @required),
+        (sort { $a->[0] cmp $b->[0] } @optional),
+        (sort { $a->[0] <=> $b->[0] } @positional),
     );
 
-    return @result;
+    return map { $_->[1] } @result;
 }
 
 
@@ -460,6 +523,9 @@ sub _shell_arg_getopt_qualifier_from_property_meta {
     if (defined($property_meta->data_type) and $property_meta->data_type =~ /Boolean/) {
         return '!' . $many;
     }
+    #elsif ($property_meta->is_optional) {
+    #    return ':s' . $many;
+    #}
     else {
         return '=s' . $many;
     }
@@ -502,7 +568,8 @@ sub _shell_arg_getopt_specification_from_property_meta {
     my $arg_name = $self->_shell_arg_name_from_property_meta($property_meta);
     return (
         $arg_name .  $self->_shell_arg_getopt_qualifier_from_property_meta($property_meta),
-        ($property_meta->is_many ? ($arg_name => []) : ())
+        #this prevents defaults from being used for is_many properties
+        #($property_meta->is_many ? ($arg_name => []) : ())
     );
 }
 
@@ -625,8 +692,7 @@ sub resolve_param_value_from_cmdline_text {
 
     my $pmeta = $self->__meta__->property($param_name);
 
-
-    print STDERR "Resolving parameter '$param_name' from command argument '$param_str'...";
+    my $param_resolve_message = "Resolving parameter '$param_name' from command argument '$param_str'...";
     my @results;
     my $require_user_verify = $pmeta->{'require_user_verify'};
     for (my $i = 0; $i < @param_args; $i++) {
@@ -651,10 +717,10 @@ sub resolve_param_value_from_cmdline_text {
         }
     }
     if (@results) {
-        print STDERR " found " . @results . ".\n";
+        $self->status_message($param_resolve_message . " found " . @results);
     }
     else {
-        print STDERR " none found.\n";
+        $self->status_message($param_resolve_message . " none found.");
     }
 
     return unless (@results);
@@ -885,8 +951,13 @@ sub _get_user_verification_for_param_value_drilldown {
     my $max_dname_length = @dnames ? length((sort { length($b) <=> length($a) } @dnames)[0]) : 0;
     my @statuses = map {$_->status} grep { $_->can('status') } @results;
     my $max_status_length = @statuses ? length((sort { length($b) <=> length($a) } @statuses)[0]) : 0;
-    @results = sort {$a->__display_name__ cmp $b->__display_name__} @results;
-    @results = sort {$a->class cmp $b->class} @results;
+
+    my @results_with_display_name_and_class = map { [ $_->__display_name__, $_->class, $_ ] } @results;
+    @results = map { $_->[2] }
+               sort { $a->[1] cmp $b->[1] }
+               sort { $a->[0] cmp $b->[0] }
+               @results_with_display_name_and_class;
+
     my @classes = $self->_unique_elements(map {$_->class} @results);
 
     my $response;
@@ -951,6 +1022,44 @@ sub _get_user_verification_for_param_value_drilldown {
     }
 }
 
+sub _ask_user_question {
+    my $self = shift;
+    my $question = shift;
+    my $timeout = shift;
+    my $valid_values = shift || "yes|no";
+    my $default_value = shift || undef;
+    my $pretty_valid_values = shift || $valid_values;
+    $valid_values = lc($valid_values);
+    my $input;
+    $timeout = 60 unless(defined($timeout));
+
+    local $SIG{ALRM} = sub { print STDERR "Exiting, failed to reply to question '$question' within '$timeout' seconds.\n"; exit; };
+    print STDERR "\n$question\n";
+    print STDERR "Reply with $pretty_valid_values: ";
+
+    unless ($self->_can_interact_with_user) {
+        print STDERR "\n";
+        die $self->error_message("Attempting to ask user question but cannot interact with user!");
+    }
+
+    alarm($timeout) if ($timeout);
+    chomp($input = <STDIN>);
+    alarm(0) if ($timeout);
+
+    print STDERR "\n";
+
+    if(lc($input) =~ /^$valid_values$/) {
+        return lc($input);
+    }
+    elsif ($default_value) {
+        return $default_value;
+    }
+    else {
+        $self->error_message("'$input' is an invalid answer to question '$question'\n\n");
+        return;
+    }
+}
+
 sub _validate_user_response_for_param_value_verification {
     my ($self, $response_text) = @_;
     $response_text = substr($response_text, 1) if ($response_text =~ /^[+-]/);
@@ -1011,6 +1120,7 @@ sub _trim_list_from_response {
 
 sub _pad_string {
     my ($self, $str, $width, $pos) = @_;
+    $str = '' if ! defined $str;
     my $padding = $width - length($str);
     $padding = 0 if ($padding < 0);
     if ($pos && $pos eq 'suffix') {
