@@ -5,7 +5,7 @@ use warnings;
 use UR;
 use File::Basename qw/basename/;
 
-our $VERSION = "0.32"; # UR $VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 class Command::Tree {
     is => 'Command::V2',
@@ -18,12 +18,15 @@ sub resolve_class_and_params_for_argv {
     my $self = shift;
     my @argv = @_;
 
-    if ( $argv[0] and $argv[0] !~ /^\-/ 
+    if ( $argv[0] and $argv[0] !~ /^\-/
             and my $class_for_sub_command = $self->class_for_sub_command($argv[0]) ) {
         # delegate
         shift @argv;
         return $class_for_sub_command->resolve_class_and_params_for_argv(@argv);
     }
+    elsif ( @argv == 1 and $argv[0] =~ /^(\-)?\-h(elp)?$/ ) { # HELP ME!
+        return ($self, { help => 1 });
+    }
     else {
         # error
         return ($self,undef);
@@ -81,7 +84,7 @@ sub doc_help {
 
     my $command_name = $self->command_name;
     my $text;
-    
+
     # show the list of sub-commands
     $text = sprintf(
         "Sub-commands for %s:\n%s",
@@ -96,13 +99,13 @@ sub doc_help {
 sub doc_manual {
     my $self = shift;
     my $pod = $self->_doc_name_version;
-    
+
     my $manual = $self->_doc_manual_body;
     my $help = $self->help_detail;
     if ($manual or $help) {
         $pod .= "=head1 DESCRIPTION:\n\n";
 
-        my $txt = $manual || $help;        
+        my $txt = $manual || $help;
         if ($txt =~ /^\=/) {
             # pure POD
             $pod .= $manual;
@@ -114,7 +117,7 @@ sub doc_manual {
         }
     }
 
-    
+
     my $sub_commands = $self->help_sub_commands(brief => 1);
     $pod .= "=head1 SUB-COMMANDS\n\n" . $sub_commands . "\n\n";
 
@@ -125,12 +128,13 @@ sub doc_manual {
 
 sub sorted_sub_command_classes {
     no warnings;
-    my @c = shift->sub_command_classes;
-    return sort {
-            ($a->sub_command_sort_position <=> $b->sub_command_sort_position)
+    my @c = map { [ $_->sub_command_sort_position, $_ ] } shift->sub_command_classes;
+    return map { $_->[1] }
+           sort {
+            ($a->[0] <=> $b->[0])
             ||
-            ($a->sub_command_sort_position cmp $b->sub_command_sort_position)
-        } 
+            ($a->[0] cmp $b->[0])
+        }
         @c;
 }
 
@@ -171,7 +175,7 @@ sub sub_commands_table {
 
 sub _categorize_sub_commands {
     my $class = shift;
-    
+
     my @sub_command_classes = $class->sorted_sub_command_classes;
     my %categories;
     my @order;
@@ -180,7 +184,7 @@ sub _categorize_sub_commands {
         my $category = $sub_command_class->sub_command_category || '';
         unless (exists $categories{$category}) {
             if ($category) {
-                push(@order, $category) 
+                push(@order, $category)
             } else {
                 unshift(@order, '');
             }
@@ -191,7 +195,7 @@ sub _categorize_sub_commands {
 
     return (\@order, \%categories);
 }
-    
+
 sub help_sub_commands {
     my ($self, %params) = @_;
     my ($order, $categories) = $self->_categorize_sub_commands(@_);
@@ -199,7 +203,7 @@ sub help_sub_commands {
 
     no warnings;
     local $Text::Wrap::columns = 60;
-    
+
     my @full_data;
     for my $category (@$order) {
         my $sub_commands_within_this_category = $categories->{$category};
@@ -212,15 +216,15 @@ sub help_sub_commands {
                         ($_->isa('Command::Tree') ? '...' : ''), #$_->_shell_args_usage_string_abbreviated,
                         $rows[0],
                     ],
-                    map { 
-                        [ 
+                    map {
+                        [
                             '',
                             ' ',
                             $rows[$_],
                         ]
                     } (1..$#rows)
                 );
-            } 
+            }
             @$sub_commands_within_this_category;
 
         if ($category) {
@@ -230,7 +234,7 @@ sub help_sub_commands {
             if ($category =~ /\D/) {
                 # non-numeric categories show their category as a header
                 $category .= ':' if $category =~ /\S/;
-                push @full_data, 
+                push @full_data,
                     [
                         Term::ANSIColor::colored(uc($category), 'blue'),
                         '',
@@ -290,7 +294,7 @@ sub doc_sub_commands {
 }
 
 #
-# The following methods build allow a command to determine its 
+# The following methods build allow a command to determine its
 # sub-commands, if there are any.
 #
 
@@ -316,38 +320,55 @@ sub is_sub_command_delegator {
     return scalar(shift->sub_command_classes);
 }
 
+sub command_tree_source_classes {
+    # override in subclass if you want different sources
+    my $class = shift;
+    return $class;
+}
+
 sub _build_sub_command_mapping {
     my $class = shift;
     $class = ref($class) || $class;
-    
+
+    my @source_classes = $class->command_tree_source_classes;
+
     my $mapping;
     do {
         no strict 'refs';
         $mapping = ${ $class . '::SUB_COMMAND_MAPPING'};
+        if (ref($mapping) eq 'HASH') {
+            return $mapping;
+        }
     };
-    
-    unless (ref($mapping) eq 'HASH') {
+
+    for my $source_class (@source_classes) {
+        # check if this class is valid
+        eval{ $source_class->class; };
+        if ( $@ ) {
+            warn $@;
+        }
+
         # for My::Foo::Command::* commands and sub-trees
-        my $subdir = $class; 
+        my $subdir = $source_class;
         $subdir =~ s|::|\/|g;
 
         # for My::Foo::*::Command sub-trees
-        my $class_above = $class;
-        $class_above =~ s/::Command//;
-        my $subdir2 = $class_above;
-        $subdir2 =~ s|::|/|;
-        
+        my $source_class_above = $source_class;
+        $source_class_above =~ s/::Command//;
+        my $subdir2 = $source_class_above;
+        $subdir2 =~ s|::|/|g;
+
         # check everywhere
         for my $lib (@INC) {
             my $subdir_full_path = $lib . '/' . $subdir;
-            
+
             # find My::Foo::Command::*
             if (-d $subdir_full_path) {
                 my @files = glob($subdir_full_path . '/*');
                 for my $file (@files) {
                     my $basename = basename($file);
                     $basename =~ s/.pm$// or next;
-                    my $sub_command_class_name = $class . '::' . $basename;
+                    my $sub_command_class_name = $source_class . '::' . $basename;
                     my $sub_command_class_meta = UR::Object::Type->get($sub_command_class_name);
                     unless ($sub_command_class_meta) {
                         local $SIG{__DIE__};
@@ -359,11 +380,12 @@ sub _build_sub_command_mapping {
                     $sub_command_class_meta = UR::Object::Type->get($sub_command_class_name);
                     next unless $sub_command_class_name->isa("Command");
                     next if $sub_command_class_meta->is_abstract;
-                    my $name = $class->_command_name_for_class_word($basename); 
+                    next if $sub_command_class_name eq $class;
+                    my $name = $source_class->_command_name_for_class_word($basename);
                     $mapping->{$name} = $sub_command_class_name;
-                }                
+                }
             }
-            
+
             # find My::Foo::*::Command
             $subdir_full_path = $lib . '/' . $subdir2;
             my $pattern = $subdir_full_path . '/*/Command.pm';
@@ -376,13 +398,14 @@ sub _build_sub_command_mapping {
                 $last_word =~ s/.pm$// or next;
                 my $dir = File::Basename::dirname($file);
                 my $second_to_last_word = File::Basename::basename($dir);
-                my $sub_command_class_name = $class_above . '::' . $second_to_last_word . '::' . $last_word;
+                my $sub_command_class_name = $source_class_above . '::' . $second_to_last_word . '::' . $last_word;
                 next unless $sub_command_class_name->isa('Command');
                 next if $sub_command_class_name->__meta__->is_abstract;
+                next if $sub_command_class_name eq $class;
                 my $basename = $second_to_last_word;
                 $basename =~ s/.pm$//;
-                my $name = $class->_command_name_for_class_word($basename); 
-                $mapping->{$name} = $sub_command_class_name;                
+                my $name = $source_class->_command_name_for_class_word($basename);
+                $mapping->{$name} = $sub_command_class_name;
             }
         }
     }
@@ -395,21 +418,12 @@ sub sub_command_names {
     return keys %$mapping;
 }
 
-sub class_for_sub_command {
+
+
+sub _try_command_class_named {
     my $self = shift;
-    my $class = ref($self) || $self;
-    my $sub_command = shift;
 
-    return if $sub_command =~ /^\-/;
-
-    my $mapping = $class->_build_sub_command_mapping;
-    if (my $sub_command_class = $mapping->{$sub_command}) {
-        return $sub_command_class;
-    }
-
-
-    my $sub_class = join("", map { ucfirst($_) } split(/-/, $sub_command));
-    $sub_class = $class . "::" . $sub_class;
+    my $sub_class = join('::', @_);
 
     my $meta = UR::Object::Type->get($sub_class); # allow in memory classes
     unless ( $meta ) {
@@ -446,6 +460,40 @@ sub class_for_sub_command {
 }
 
 
+sub class_for_sub_command {
+    my $self = shift;
+    my $class = ref($self) || $self;
+    my $sub_command = shift;
+
+    return if $sub_command =~ /^\-/;  # If it starts with a "-", then it's a command-line option
+
+    # First attempt is to convert $sub_command into a camel-case module name
+    # and just try loading it
+
+    my $name_for_sub_command = join("", map { ucfirst($_) } split(/-/, $sub_command));
+    my @class_name_parts = (split(/::/,$class), $name_for_sub_command);
+    my $sub_command_class = $self->_try_command_class_named(@class_name_parts);
+    return $sub_command_class if $sub_command_class;
+
+    # Remove "Command" if it's embedded in the middle and try inserting it in other places, starting at the end
+    @class_name_parts = ( ( map { $_ eq 'Command' ? () : $_ } @class_name_parts) , 'Command');
+    for(my $i = $#class_name_parts; $i > 0; $i--) {
+        $sub_command_class = $self->_try_command_class_named(@class_name_parts);
+        return $sub_command_class if $sub_command_class;
+        $class_name_parts[$i] = $class_name_parts[$i-1];
+        $class_name_parts[$i-1] = 'Command';
+    }
+
+    # Didn't find it yet.  Try exhaustively loading all the command modules under $class
+    my $mapping = $class->_build_sub_command_mapping;
+    if (my $sub_command_class = $mapping->{$sub_command}) {
+        return $sub_command_class;
+    } else {
+        return;
+    }
+}
+
+
 1;
 
 __END__
@@ -454,7 +502,7 @@ __END__
 
 =head1 NAME
 
-Command::Tree -base class for commands which delegate to a list of sub-commands 
+Command::Tree -base class for commands which delegate to a list of sub-commands
 
 =head1 DESCRIPTION
 
