@@ -1,10 +1,7 @@
-package Command::V2;  # additional methods to produce documentation
+package Command::V2;  # additional methods to produce documentation, TODO: turn into a real view
 use strict;
 use warnings;
 
-our $entry_point_class;
-our $entry_point_bin;
-
 use Term::ANSIColor;
 use Pod::Simple::Text;
 require Text::Wrap;
@@ -35,7 +32,6 @@ sub help_brief {
     }
 }
 
-
 sub help_synopsis {
     my $self = shift;
     return '';
@@ -88,7 +84,7 @@ sub doc_help {
     my $required_args = $self->help_options(is_optional => 0);
     my $optional_args = $self->help_options(is_optional => 1);
     $text = sprintf(
-        "\n%s\n%s\n\n%s%s%s%s%s%s\n",
+        "\n%s\n%s\n\n%s%s%s%s%s\n",
         Term::ANSIColor::colored('USAGE', 'underline'),
         Text::Wrap::wrap(
             ' ', 
@@ -427,7 +423,7 @@ sub help_options {
         else {
             $text .= sprintf(
                 "  %s\n%s\n",
-                Term::ANSIColor::colored($row->[0], 'bold') . "   " . $row->[1],
+                Term::ANSIColor::colored($row->[0], 'bold'), # . "   " . $row->[1],
                 Text::Wrap::wrap(
                     "    ", # 1st line indent,
                     "    ", # all other lines indent,
@@ -499,30 +495,27 @@ sub _doc_see_also {
 sub _shell_args_usage_string {
     my $self = shift;
 
-    my $is_executable = eval {
-        if ($self->can("_execute_body") eq __PACKAGE__->can("_execute_body")) {
-            return;
+    return eval {
+        print $self->class."\n";
+        if ( $self->isa('Command::Tree') ) { 
+            return '...';
+        }
+        elsif ($self->can("_execute_body") eq __PACKAGE__->can("_execute_body")) {
+            return '(no execute!)';
         }
         elsif ($self->__meta__->is_abstract) {
-            return;
+            return '(no sub commands!)';
         }
         else {
-            return 1;
+            return join(
+                " ", 
+                map { 
+                    $self->_shell_arg_usage_string_from_property_meta($_) 
+                } $self->_shell_args_property_meta()
+
+            );
         }
     };
-
-    if ($is_executable) {
-        return join(
-            " ", 
-            map { 
-                $self->_shell_arg_usage_string_from_property_meta($_) 
-            } $self->_shell_args_property_meta()
-            
-        );
-    }
-    else {
-        return "(no execute or sub commands implemented)"
-    }
 }
 
 sub _shell_args_usage_string_abbreviated {
@@ -540,8 +533,8 @@ sub command_name {
     my $self = shift;
     my $class = ref($self) || $self;
     my $prepend = '';
-    if (defined($entry_point_class) and $class =~ /^($entry_point_class)(::.+|)$/) {
-        $prepend = $entry_point_bin;
+    if (defined($Command::entry_point_class) and $class =~ /^($Command::entry_point_class)(::.+|)$/) {
+        $prepend = $Command::entry_point_bin;
         $class = $2;
         if ($class =~ s/^:://) {
             $prepend .= ' ';
@@ -586,7 +579,7 @@ sub _command_name_for_class_word {
     $s =~ s/_/-/g;
     $s =~ s/^([A-Z])/\L$1/; # ignore first capital because that is assumed
     $s =~ s/([A-Z])/-$1/g; # all other capitals prepend a dash
-    $s =~ s/([a-zA-Z])([0-9])/$1-$2/g; # treat number as begining word
+    $s =~ s/([a-zA-Z])([0-9])/$1$2/g; # treat number as begining word
     $s = lc($s);
     return $s;
 }
