@@ -3,7 +3,7 @@ package UR::All;
 use strict;
 use warnings;
 
-our $VERSION = "0.32"; # UR $VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 BEGIN { require above; };
 use UR;
@@ -68,7 +68,6 @@ use UR::DataSource::RDBMS::Table::View::Default::Text;
 use UR::DataSource::RDBMS::TableColumn;
 use UR::DataSource::RDBMS::TableColumn::View::Default::Text;
 use UR::DataSource::RDBMS::UniqueConstraintColumn;
-use UR::DataSource::RemoteCache;
 use UR::DataSource::SQLite;
 use UR::DataSource::ValueDomain;
 use UR::DBI;
@@ -153,12 +152,6 @@ use UR::Namespace::Command::Update::RenameClass;
 use UR::Namespace::Command::Update::RewriteClassHeader;
 use UR::Namespace::Command::Update::SchemaDiagram;
 use UR::Namespace::Command::Update::TabCompletionSpec;
-use UR::Namespace::View::SchemaBrowser::CgiApp;
-use UR::Namespace::View::SchemaBrowser::CgiApp::Base;
-use UR::Namespace::View::SchemaBrowser::CgiApp::Class;
-use UR::Namespace::View::SchemaBrowser::CgiApp::File;
-use UR::Namespace::View::SchemaBrowser::CgiApp::Index;
-use UR::Namespace::View::SchemaBrowser::CgiApp::Schema;
 use UR::Object;
 use UR::Object::Accessorized;
 use UR::Object::Command::FetchAndDo;
@@ -173,7 +166,6 @@ use UR::Object::Property::View::DescriptionLineItem::Text;
 use UR::Object::Property::View::ReferenceDescription::Text;
 use UR::Object::Set;
 use UR::Object::Set::View::Default::Json;
-use UR::Object::Set::View::Default::Xml;
 use UR::Object::Tag;
 use UR::Object::Type;
 use UR::Object::Type::AccessorWriter;
@@ -183,17 +175,13 @@ use UR::Object::Type::Initializer;
 use UR::Object::Type::InternalAPI;
 use UR::Object::Type::ModuleWriter;
 use UR::Object::Type::View::Default::Text;
-use UR::Object::Type::View::Default::Xml;
 use UR::Object::Value;
 use UR::Object::View;
 use UR::Object::View::Aspect;
 use UR::Object::View::Default::Gtk;
 use UR::Object::View::Default::Gtk2;
-use UR::Object::View::Default::Html;
 use UR::Object::View::Default::Json;
 use UR::Object::View::Default::Text;
-use UR::Object::View::Default::Xml;
-use UR::Object::View::Default::Xsl;
 use UR::Object::View::Lister::Text;
 use UR::Object::View::Toolkit;
 use UR::Object::View::Toolkit::Text;
@@ -201,9 +189,7 @@ use UR::ObjectDeprecated;
 use UR::ObjectV001removed;
 use UR::ObjectV04removed;
 use UR::Observer;
-use UR::Report;
-use UR::Service::DataSourceProxy;
-use UR::Service::JsonRpcServer;
+use UR::DBI::Report;
 use UR::Service::RPC::Executer;
 use UR::Service::RPC::Message;
 use UR::Service::RPC::Server;
@@ -232,6 +218,32 @@ use UR::Value::Text;
 use UR::Value::URL;
 use UR::Vocabulary;
 
+# optional elements
+if (eval "use Net::HTTPServer") {
+    my $rv = eval "UR::Namespace::View::SchemaBrowser::CgiApp;"
+             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::Base;"
+             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::Class;"
+             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::File;"
+             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::Index;"
+             && eval "use UR::Namespace::View::SchemaBrowser::CgiApp::Schema;"
+             && eval "use UR::Service::JsonRpcServer;";
+    die $@ unless ($rv);
+}
+
+
+if (eval "use Xml::LibXSLT") {
+    my $rv = eval "use UR::Object::View::Default::Html;"
+             && eval "use UR::Object::View::Default::Xsl;"
+             && eval "use UR::Object::Set::View::Default::Xml;"
+             && eval "use UR::Object::View::Default::Xml;"
+             && eval "use UR::Object::Type::View::Default::Xml;"
+             ;
+    die $@ unless ($rv);
+}
+ 
+
+1;
+
 __END__
 
 =pod
