@@ -4,19 +4,15 @@ package UR::BoolExpr::Template::PropertyComparison::NotEqual;
 use strict;
 use warnings;
 use UR;
-our $VERSION = $UR::VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name  => __PACKAGE__, 
     is => ['UR::BoolExpr::Template::PropertyComparison'],
 );
 
-sub evaluate_subject_and_values {
-    my $self = shift;
-    my $subject = shift;
-    my $comparison_value = shift;    
-    my $property_name = $self->property_name;    
-    my @property_value = $subject->$property_name;
+sub _compare {
+    my ($class,$comparison_value,@property_value) = @_;
 
     no warnings 'uninitialized';
     if (@property_value == 0) {
@@ -44,7 +40,7 @@ sub evaluate_subject_and_values {
 
 =head1 NAME
 
-UR::BoolExpr::Template::PropertyComparison::NotEqual - Perform a not-equal test
+UR::BoolExpr::Template::PropertyComparison::NotEqual - perform a not-equal test
 
 =head1 DESCRIPTION
 
