@@ -7,6 +7,7 @@ use warnings;
 use IO::File;
 
 require UR;
+our $VERSION = "0.38"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
@@ -55,6 +56,7 @@ sub undo {
     if ($changed_aspect eq "delete" or $changed_aspect eq "unload") {
         $undo_data = '' unless defined $undo_data;
         $changed_obj = eval "no strict; no warnings; " . $undo_data;
+        bless($changed_obj, 'UR::DeletedRef') if (ref $changed_obj); # changed class so that UR::Object::DESTROY is not called on a "fake" UR::Object
         if ($@) {
             Carp::confess("Error reconstructing $changed_aspect data for @_: $@");
         }
@@ -70,7 +72,11 @@ sub undo {
         UR::Object::unload($changed_obj);
     }
     elsif ($changed_aspect eq "create") {
-        UR::Object::delete($changed_obj);
+        if ($changed_obj->isa('UR::Observer')) {
+            UR::Observer::delete($changed_obj);  # Observers have state that needs to be cleaned up
+        } else {
+            UR::Object::delete($changed_obj);
+        }
     }
     elsif ($changed_aspect eq "delete") {
         my %stored;
@@ -85,6 +91,7 @@ sub undo {
         for my $key (keys %stored) {
             $changed_obj->{$key} = $stored{$key};
         }
+        $changed_obj->{'_change_count'}--;  # it was incremented when delete() was called on the object
     }
     elsif ($changed_aspect eq "load") {
         UR::Object::unload($changed_obj);
@@ -126,9 +133,12 @@ sub undo {
         # regular property
         if ($changed_obj->can($changed_aspect)) {
             $changed_obj->$changed_aspect($undo_data);
+            $changed_obj->{'_change_count'} -= 2;  # 2 because the line above will actually increment the counter, too
         }
     }
 
+    $changed_obj->{'_change_count'} = 0 if ($changed_obj->{'_change_count'} and $changed_obj->{'_change_count'} < 0);
+
     return 1;
 }
 
