@@ -28,7 +28,7 @@ This module subclasses DBI, and provides a few extra methods useful when using a
 require 5.006_000;
 use warnings;
 use strict;
-our $VERSION = "0.32"; # UR $VERSION;;
+our $VERSION = "0.38"; # UR $VERSION;;
 
 # set up module
 use base qw(Exporter DBI);
@@ -322,6 +322,9 @@ sub before_execute
 	$monitor_dml=2;
     }
     no warnings;            
+    
+    UR::DBI::log_sql_for_summary($sql);    # $ENV{UR_DBI_SUMMARIZE_SQL}
+
     my $log_sql_str = _generate_sql_and_params_log_entry($sql, @_);
     UR::DBI::log_sql($log_sql_str);
     return $start_time;
@@ -446,11 +449,25 @@ sub after_all_fetches_no_sth {
 }
 
 
+my $__SQL_SUMMARY__ = {};
+sub log_sql_for_summary {
+    my ($sql) = @_;
+    $__SQL_SUMMARY__->{$sql}++; 
+}
+
+sub print_sql_summary {
+    for my $sql (sort {$__SQL_SUMMARY__->{$b} <=> $__SQL_SUMMARY__->{$a}} keys %$__SQL_SUMMARY__) {
+        print STDERR join('',"********************\n", $__SQL_SUMMARY__->{$sql}, " instances of query: $sql\n");
+    }
+}
+
 # These methods are called by the above.
 
 sub _generate_sql_and_params_log_entry 
 {
+
     my $sql = shift;
+
     no warnings;
     my $sql_log_str =  "\nSQL: $sql\n"; 
     if (@_) {
@@ -550,7 +567,7 @@ sub _print_query_plan
     
     $dbh->do($UR::DBI::EXPLAIN_PLAN_DML . "\n" . $sql)
         or die "Failed to produce query plan! " . $dbh->errstr;        
-    UR::Report->generate(
+    UR::DBI::Report->generate(
         sql => [$UR::DBI::EXPLAIN_PLAN_SQL],
         dbh => $dbh,
         count => 0,
@@ -662,7 +679,7 @@ sub disconnect
     
     # Msg and disconnect.
     UR::DBI::before_execute("disconnecting");
-    $self->SUPER::disconnect(@_);
+    my $rv = $self->SUPER::disconnect(@_);
     UR::DBI::after_execute();
     
     # There doesn't seem to be anything less which
@@ -677,6 +694,7 @@ sub disconnect
         $UR::DBI::common_dbh = undef;
         UR::DBI::after_execute("common dbh removed");
     }
+    return $rv;
 }
 
 sub prepare
