@@ -4,11 +4,11 @@ use warnings;
 package UR::DataSource::RDBMS::FkConstraint;
 
 use UR;
-our $VERSION = $UR::VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
+
 UR::Object::Type->define(
     class_name => 'UR::DataSource::RDBMS::FkConstraint',
     is => ['UR::DataSource::RDBMS::Entity'],
-    english_name => 'dd fk constraint',
     dsmap => 'dd_fk_constraint',
     er_role => '',
     id_properties => [qw/data_source owner r_owner table_name r_table_name fk_constraint_name/],
@@ -48,7 +48,7 @@ sub _table_classes {
 sub get_with_special_params {
     my($class,$rule,%args) = @_;
 
-$DB::single=1;
+#$DB::single = 1;
     my $column_name = delete $args{'column_name'};
     my $r_column_name = delete $args{'r_column_name'};
 
@@ -174,7 +174,7 @@ my $self = shift;
 
 =head1 NAME 
 
-UR::DataSource::RDBMS::FkConstraint - Metadata about a data source's foreign keys
+UR::DataSource::RDBMS::FkConstraint - metadata about a data source's foreign keys
 
 =head1 DESCRIPTION
 
