@@ -4,11 +4,11 @@ use warnings;
 package UR::DataSource::RDBMS::FkConstraintColumn;
 
 use UR;
-our $VERSION = $UR::VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
+
 UR::Object::Type->define(
     class_name => 'UR::DataSource::RDBMS::FkConstraintColumn',
     is => ['UR::DataSource::RDBMS::Entity'],
-    english_name => 'dd fk constraint column',
     dsmap => 'dd_fk_constraint_column',
     er_role => 'bridge',
     id_properties => [qw/data_source owner table_name fk_constraint_name column_name/],
@@ -33,7 +33,7 @@ UR::Object::Type->define(
 
 =head1 NAME 
 
-UR::DataSource::RDBMS::FkConstraintColumn - Metadata about a data source's foreign keys
+UR::DataSource::RDBMS::FkConstraintColumn - metadata about a data source's foreign keys
 
 =head1 DESCRIPTION
 
