@@ -3,12 +3,12 @@ use warnings;
 
 package UR::DataSource::RDBMS::UniqueConstraintColumn;
 
-use UR::Object::Type;
-our $VERSION = $UR::VERSION;
+use UR;
+our $VERSION = "0.38"; # UR $VERSION;
+
 UR::Object::Type->define(
     class_name => 'UR::DataSource::RDBMS::UniqueConstraintColumn',
     is => ['UR::DataSource::RDBMS::Entity'],
-    english_name => 'dd_unique_constraint_column',
     dsmap => 'dd_unique_constraint_column',
     id_properties => [qw/data_source owner table_name constraint_name column_name/],
     properties => [
@@ -31,7 +31,7 @@ UR::Object::Type->define(
 
 =head1 NAME
 
-UR::DataSource::RDBMS::UniqueConstraintColumn - Metadata about a data source's unique constraints
+UR::DataSource::RDBMS::UniqueConstraintColumn - metadata about a data source's unique constraints
 
 =head1 DESCRIPTION
 
