@@ -21,67 +21,20 @@ BEGIN {
 
 =head1 NAME
 
-UR::ModuleBase - Error, status, and warning messaging for derived packages
+UR::ModuleBase - Methods common to all UR classes and object instances.
 
-=head1 SYNOPSIS
-
-    # common usage
-    
-    sub foo {
-        my $self = shift;
-        ...
-        if ($problem) {           
-            $self->error_message("Something went wrong...");
-            return;
-        }
-        return 1;
-    }
-    
-    unless ($obj->foo) {
-        print STDERR $obj->error_string;
-    }
-    
-    # A complete object is made with each error, staus,
-    # or warning message with detail about the context in 
-    # which it was created.
-    
-    # Some of the details are accessible directly on the object/class:
-    $string       = $obj->error_string;
-    $text         = $obj->error_text;
-    $package_name = $obj->error_package;
-    @call_stack   = $obj->error_call_stack;
-    $time         = $obj->error_time;
-
-    # The developer can also get the message object directly
-    # and examine the properties.
-    $msg_obj      = $obj->error_object;
-        $string         = $msg_obj->string; 
-        $text           = $msg_obj->text;
-        $package_name   = $msg_obj->package_name;
-        @call_stack     = $msg_obj->call_stack;
-        $time           = $msg_obj->time;
-        $type           = $msg_obj->type;  # "error"
-        $owner          = $msg_obj->owner; # $obj
-
-    # WARNING: error_message will return the last error specified
-    # on that object/class.  It is not automatically reset when
-    # other methods are called which work without error.
-    
-    # When no error is explicitly found, a check is made against
-    # the last error in general (UR::ModuleBase->error_message)
-    # and call stacks are compared.  If it occurred in something    
-    # called by the caller.
-    
-    
 =head1 DESCRIPTION
 
 This is a base class for packages, classes, and objects which need to
-set/get error, warning, debug, and status messages on themselves,
-their class, and their parent class(es).
+manage basic functionality in the UR framework such as inheritance, 
+AUTOLOAD/AUTOSUB methods, error/status/warning/etc messages.
+
+UR::ModuleBase is in the @ISA list for UR::Object, but UR::ModuleBase is not
+a formal UR class.
 
 =head1 METHODS
 
-These methods create and change message handlers.
+=over 4
 
 =cut
 
@@ -89,7 +42,7 @@ These methods create and change message handlers.
 require 5.006_000;
 use warnings;
 use strict;
-our $VERSION = "0.32"; # UR $VERSION;;
+our $VERSION = "0.38"; # UR $VERSION;;
 
 # set up module
 use Carp;
@@ -102,7 +55,7 @@ use UR::Util;
 
   $class = $obj->class;
 
-This returns the class name of a class or an object.
+This returns the class name of a class or an object as a string.
 It is exactly equivalent to:
 
     (ref($self) ? ref($self) : $self)
@@ -247,14 +200,79 @@ sub base_dir
     my $class = ref($self) || $self;    
     $class =~ s/\:\:/\//g;
     my $dir = $INC{$class . '.pm'} || $INC{$class . '.pl'};
-    die "Failed to find module $class in \%INC: " . Dumper(%INC) unless ($dir);
+    die "Failed to find module $class in \%INC: " . Data::Dumper(%INC) unless ($dir);
     $dir =~ s/\.p[lm]\s*$//;
     return $dir;
 }
 
 =pod
 
-=item C<AUTOLOAD>
+=item methods
+
+Undocumented.
+
+=cut
+
+sub methods
+{
+    my $self = shift;
+    my @methods;
+    my %methods;
+    my ($class, $possible_method, $possible_method_full, $r, $r1, $r2);
+    no strict; 
+    no warnings;
+
+    for $class (reverse($self, $self->inheritance())) 
+    { 
+        print "$class\n"; 
+        for $possible_method (sort grep { not /^_/ } keys %{$class . "::"}) 
+        {
+            $possible_method_full = $class . "::" . $possible_method;
+            
+            $r1 = $class->can($possible_method);
+            next unless $r1; # not implemented
+            
+            $r2 = $class->super_can($possible_method);
+            next if $r2 eq $r1; # just inherited
+            
+            {
+                push @methods, $possible_method_full; 
+                push @{ $methods{$possible_method} }, $class;
+            }
+        } 
+    }
+    print Dumper(\%methods);
+    return @methods;
+}
+
+=pod
+
+=item C<context_return>
+
+  return MyClass->context_return(@return_values);
+
+Attempts to return either an array or scalar based on the calling context.
+Will die if called in scalar context and @return_values has more than 1
+element.
+
+=cut
+
+sub context_return {
+    my $class = shift;
+    return unless defined wantarray;
+    return @_ if wantarray;
+    if (@_ > 1) {
+        my @caller = caller(1);
+        Carp::croak("Method $caller[3] on $class called in scalar context, but " . scalar(@_) . " items need to be returned");
+    }
+    return $_[0];
+}
+
+=pod
+
+=back
+
+=head1 C<AUTOLOAD>
 
 This package impliments AUTOLOAD so that derived classes can use
 AUTOSUB instead of AUTOLOAD.
@@ -372,71 +390,33 @@ sub AUTOLOAD {
     }
 }
 
-=pod
-
-=item methods
-
-Undocumented.
-
-=cut
-
-sub methods
-{
-    my $self = shift;
-    my @methods;
-    my %methods;
-    my ($class, $possible_method, $possible_method_full, $r, $r1, $r2);
-    no strict; 
-    no warnings;
-
-    for $class (reverse($self, $self->inheritance())) 
-    { 
-        print "$class\n"; 
-        for $possible_method (sort grep { not /^_/ } keys %{$class . "::"}) 
-        {
-            $possible_method_full = $class . "::" . $possible_method;
-            
-            $r1 = $class->can($possible_method);
-            next unless $r1; # not implemented
-            
-            $r2 = $class->super_can($possible_method);
-            next if $r2 eq $r1; # just inherited
-            
-            {
-                push @methods, $possible_method_full; 
-                push @{ $methods{$possible_method} }, $class;
-            }
-        } 
-    }
-    print Dumper(\%methods);
-    return @methods;
-}
 
 =pod
 
-=item C<context_return>
+=head1 MESSAGING
 
-  return MyClass->context_return(@return_values);
+UR::ModuleBase implements several methods for sending and storing error, warning and
+status messages to the user.  
 
-Attempts to return either an array or scalar based on the calling context.
-Will die if called in scalar context and @return_values has more than 1
-element.
+  # common usage
 
-=cut
+  sub foo {
+      my $self = shift;
+      ...
+      if ($problem) {
+          $self->error_message("Something went wrong...");
+          return;
+      }
+      return 1;
+  }
 
-sub context_return {
-    my $class = shift;
-    return unless defined wantarray;
-    return @_ if wantarray;
-    if (@_ > 1) {
-        my @caller = caller(1);
-        Carp::croak("Method $caller[3] on $class called in scalar context, but " . scalar(@_) . " items need to be returned");
-    }
-    return $_[0];
-}
+  unless ($obj->foo) {
+      print LOG $obj->error_message();
+  }
 
+=head2 Messaging Methods
 
-=over
+=over 4
 
 =item message_types
 
@@ -447,12 +427,16 @@ With no arguments, this method returns all the types of messages that
 this class handles.  With arguments, it adds a new type to the
 list.
 
+Standard message types are error, status, warning, debug and usage.
+
 Note that the addition of new types is not fully supported/implemented
 yet.
 
+=back
+
 =cut
 
-our @message_types = qw(error status warning debug);
+our @message_types = qw(error status warning debug usage);
 sub message_types
 {
     my $class = shift;
@@ -463,187 +447,303 @@ sub message_types
     return @message_types;
 }
 
-# create methods to set and return messages
-foreach my $type (@message_types)
-{
-    no strict 'refs';
-    # This method looks like a r/w accessor, but internally does extra work.
-    # On write it actually creates a message object from the passed-in text (and call stack info).
-    # On read it, retrieves such object, if it exists, and returns the ->text property of it.
-    # Other methods below allow deeper introspection to the last message logged for an object/class. 
-    my $accessor = sub {
-        my $msg = shift->message_object($type, @_);
-        return ($msg ? $msg->text : undef);
-    };
-    *{"${type}_message"} = $accessor; 
+#
+# Implement error_mesage/warning_message/status_message in a way
+# which handles object-specific callbacks.
+#
+# Build a set of methods for getting/setting/printing error/warning/status messages
+# $class->dump_error_messages(<bool>) Turn on/off printing the messages to STDERR
+#     error and warnings default to on, status messages default to off
+# $class->queue_error_messages(<bool>) Turn on/off queueing of messages
+#     defaults to off
+# $class->error_message("blah"): set an error message
+# $class->error_message() return the last message
+# $class->error_messages()  return all the messages that have been queued up
+# $class->error_messages_arrayref()  return the reference to the underlying
+#     list messages get queued to.  This is the method for truncating the list
+#     or altering already queued messages
+# $class->error_messages_callback(<subref>)  Specify a callback for when error
+#     messages are set.  The callback runs before printing or queueing, so
+#     you can alter @_ and change the message that gets printed or queued
+# And then the same thing for status and warning messages
 
-    # methods to access different features of the message
-    foreach my $func_suffix (qw(string text package_name call_stack time_stamp
-                                level))
-    {
-        # This is used in the closure below.
-        # It must be lexically scoped INSIDE of the for loop.
-        my $mobj_method = $func_suffix;
+# The filehandle to print these messages to.  In normal operation this'll just be
+# STDERR, but the test case can change it to capture the messages to somewhere else
 
-        # Set the class method up.
-        my $fname = "${type}_${func_suffix}";
-        *$fname = sub
-        {
-            my $self = shift;
-            my $message_object
-                = (ref($self) ? $self->{"${type}_message"} : ${"${self}::${type}_message"});
-            return unless $message_object;
-            return $message_object->$mobj_method(@_);
+our $stderr = \*STDERR;
+our $stdout = \*STDOUT;
+
+our %msgdata;
+
+sub _get_msgdata {
+    my $self = $_[0];
+
+    if (ref($self)) {
+        no strict 'refs';
+        my $class_msgdata = ref($self)->_get_msgdata;
+        my $object_msgdata = $class_msgdata->{'__by_id__'}->{$self->id} ||= {};
+
+        while (my ($k,$v) = each(%$class_msgdata)) {
+            # Copy class' value for this config item unless it's already set on the object
+            $object_msgdata->{$k} = $v unless (exists $object_msgdata->{$k});
         }
+
+        return $object_msgdata;
+
+    } elsif ($self eq 'UR::Object') {
+        $UR::Object::msgdata ||= {};
+        return $UR::Object::msgdata;
+    }
+    else {
+        no strict 'refs';
+        my $class_msgdata = ${ $self . '::msgdata' } ||= {};
+
+        # eval since some packages aren't forman UR classes with metadata
+        my $parent_class = eval { @{$self->__meta__->is}[0]; };  # yeah, ignore multiple inheritance :(
+        my $parent_msgdata;
+        if ($parent_class) {
+            $parent_msgdata = $parent_class->_get_msgdata();
+        } else {
+            $parent_msgdata = UR::Object->_get_msgdata();
+        }
+
+        while (my ($k,$v) = each(%$parent_msgdata)) {
+            # Copy class' value for this config item unless it's already set on the object
+            $class_msgdata->{$k} = $v unless (exists $class_msgdata->{$k});
+        }
+        return $class_msgdata;
     }
 }
 
 =pod
 
-=item message_callback
+For each message type, several methods are created for sending and retrieving messages,
+registering a callback to run when messages are sent, controlling whether the messages
+are printed on the terminal, and whether the messages are queued up.
 
-  $sub_ref = UR::ModuleBase->message_callback($type);
-  UR::ModuleBase->message_callback($type, $sub_ref);
+For example, for the "error" message type, these methods are created:
 
-This method returns and optionally sets the subroutine that handles
-messages of a specific type.
+=over 4
+
+=item error_message
+
+    $obj->error_message("Something went wrong...");
+    $msg = $obj->error_message();
+
+When called with one argument, it sends an error message to the object.  The 
+error_message_callback will be run, if one is registered, and the message will
+be printed to the terminal.  When called with no arguments, the last message
+sent will be returned.  If the message is C<undef> then no message is printed
+or queued, and the next time error_message is run as an accessor, it will
+return undef.
+
+=item dump_error_messages
+
+    $obj->dump_error_messages(0);
+    $flag = $obj->dump_error_messages();
+
+Get or set the flag which controls whether messages sent via C<error_message()>
+is printed to the terminal.  This flag defaults to true for warning and error
+messages, and false for others.
+
+=item queue_error_messages
+
+    $obj->queue_error_messages(0);
+    $flag = $obj->queue_error_messages();
+
+Get or set the flag which control whether messages send via C<error_message()>
+are saved into a list.  If true, every message sent is saved and can be retrieved
+with L<error_messages()> or L<error_messages_arrayref()>.  This flag defaults to
+false for all message types.
+
+=item error_messages_callback
+
+    $obj->error_messages_callback($subref);
+    $subref = $obj->error_messages_callback();
+
+Get or set the callback run whenever an error_message is sent.  This callback
+is run with two arguments: The object or class error_message() was called on,
+and a string containing the message.  This callback is run before the message
+is printed to the terminal or queued into its list.  The callback can modify
+the message (by writing to $_[1]) and affect the message that is printed or
+queued.  If $_[1] is set to C<undef>, then no message is printed or queued,
+and the last recorded message is set to undef as when calling error_message
+with undef as the argument.
+
+=item error_messages
+
+    @list = $obj->error_messages();
+
+If the queue_error_messages flag is on, then this method returns the entire list
+of queued messages.
+
+=item error_messages_arrayref
+
+    $listref = $obj->error_messages_arrayref();
+
+If the queue_error_messages flag is on, then this method returns a reference to
+the actual list where messages get queued.  This list can be manipulated to add
+or remove items.
+
+=item error_message_source
+
+    %source_info = $obj->error_message_source
+
+Returns a hash of information about the most recent call to error_message.
+The key "error_message" contains the message.  The keys error_package,
+error_file, error_line and error_subroutine contain info about the location
+in the code where error_message() was called.
+
+=item error_package
+
+=item error_file
+
+=item error_line
+
+=item error_subroutine
+
+These methods return the same data as $obj->error_message_source().
+
+=back
 
 =cut
 
-# set or return a callback that has been created for a message type
-our %message_callback;
-sub message_callback
-{
-    my $self = shift;
-    my ($type, $callback) = @_;
+for my $type (@message_types) {
 
-    # set the callback for a given message type if callback provided
-    if (@_ > 1)
-    {
-        $message_callback{$type} = $callback;
-    }
-    return $message_callback{$type};
-}
-
-# create message object, fire callbacks, and set message on parents
-sub message_object
-{
-    my $self = shift;
-    # see how we were called
-    if (@_ < 2)
-    {
-        no strict 'refs';
-        # return the message object
-        my ($type) = @_;
-        my $mobj = (ref($self) ? $self->{"${type}_message"} : ${"${self}::${type}_message"});
-        unless ($mobj) 
-        {
-            # See if the most recent message was directly or 
-            # indirectly on $self in our current scope.
-            my $last_message = ${"UR::ModuleBase::${type}_message"};
-            if ($last_message) 
-            {
-                # Get both call stacks.
-                my @s1 = $last_message->call_stack;
-                my $s2 = _current_call_stack();
-                                                
-                # Make sure the upper call of the last error
-                # matches all of our current upper call stack.
-                for (my $n=$#$s2-1; $n >=0; $n--) {
-                    if ($s1[$n] ne $s2->[$n]) {
-                        # call stack mismatch
-                        return;
-                    }
-                }                
-                
-                # Make sure that the last message 
-                # occurred under a call to $self. 
-                my $last_message_sub_at_this_scope = $s1[$#$s2];
-                unless ($last_message_sub_at_this_scope) {
-                    return;
-                }
-                my ($pkg,$sub) = 
-                    ($last_message_sub_at_this_scope =~ /^\s*(\w+)::([^\:\W]+)/);
-                unless ($self->isa($pkg) and $self->can($sub)) {
-                    # The last message did not occur under this object.
-                    return;
-                }
-                
-                # No mismatch.  The message occurred in the current function,
-                # and under the object we're testing.  Steal the message.
-                $mobj = $last_message;
-                $s1[-1] =~ s/^(\s+).* called at/ logged at/;
-                $s1[-1] = $1 . ucfirst($type) . $s1[-1];
-                $self->message_object (
-                    "warning",
-                    "Found unpropagated error message.  Set $type message before return from: ${pkg}::${sub}:\n"
-                        . join("\n",@s1)
-                );
-            }            
-        }
-        return $mobj;
-    }
-    else
-    {
-        # create a message object
-        my ($type, $text, $level) = @_;
-        $text ||= '(not set)';
-        $level ||= 1;
-        my $class = $self->class;
-        my $id = (ref($self) ? ($self->can("id") ? $self->id : $self) : $self);
-
-        # Turn the message into an object with all of the goodies.
-        my $message_object = UR::ModuleBase::Message->create
-        (
-            text         => $text,
-            level        => $level,
-            package_name => ((caller(0))[0]),
-            call_stack   => ($type eq "error" ? _current_call_stack() : []),
-            time_stamp   => time,
-            type         => $type,
-            owner_class  => $class,
-            owner_id     => $id,
+    for my $method_base (qw/_messages_callback queue_ dump_ _package _file _line _subroutine/) {
+        my $method = (substr($method_base,0,1) eq "_"
+            ? $type . $method_base
+            : $method_base . $type . "_messages"
         );
-
-        no strict 'refs';
-
-        # Get existing values for the class and object.
-        my $class_old = ${"${class}::${type}_message"};
-        my $object_old =  $self->{"${type}_message"} if (ref($self));
-
-        # Fire the callback as appropriate.
-        my $callback = $message_callback{$type};
-        if ($callback)
-        {
-            $callback->($message_object,$self,$type,$class_old,$object_old);
-        }
-        elsif ($type eq 'status') {
-            my $t = $text;
-            chomp($t);
-            STDERR->print($t,"\n");
-        }
-        elsif ($type ne 'debug')
-        {
-            # if no callback defined, print non-debug messages to stderr
-            my $warn_txt = "$class: " . uc($type) . ": "
-                . join(': ', (caller(2))[0, 2]) . ": $text";
-            chomp($warn_txt);
-            STDERR->print("$warn_txt\n");
-        }
-
-        # Set the value on the object, the class, and all parent classes.
-        $self->{"${type}_message"} = $message_object if (ref($self));
-        foreach my $set_class ($class, $class->inheritance)
-        {
-            no warnings;
-            ${"${set_class}::${type}_message"} = $message_object;
-        }
-
-        # Return the message which was passed-in.
-        return $message_object;
+        my $method_subref = sub {
+            my $self = shift;
+            my $msgdata = $self->_get_msgdata;
+            $msgdata->{$method} = pop if @_;
+            return $msgdata->{$method};
+        };
+        no strict;
+        no warnings;
+        *$method = $method_subref;
     }
+
+    my $logger_subname = $type . "_message";
+    my $logger_subref = sub {
+        my $self = shift;
+        my @messages = @_;
+
+        my $msgdata = $self->_get_msgdata();
+
+        if (@messages > 1) {
+            Carp::carp("More than one string passed to ".$type."_message; only using the first one");
+        }
+        if (@messages) {
+            my $msg = shift @messages;
+            chomp $msg if defined $msg;
+
+            unless (defined ($msgdata->{'dump_' . $type . '_messages'})) {
+                my $do_dump;
+                if ($type eq "status" 
+                    and
+                    exists $ENV{'UR_COMMAND_DUMP_STATUS_MESSAGES'}
+                    and
+                    $ENV{'UR_COMMAND_DUMP_STATUS_MESSAGES'}
+                ) {
+                    $do_dump = 1;
+                } elsif ($type eq 'debug'
+                    and
+                    exists $ENV{'UR_DUMP_DEBUG_MESSAGES'}
+                    and
+                    $ENV{'UR_DUMP_DEBUG_MESSAGES'}
+                ) {
+                    $do_dump = 1;
+                } elsif ($type eq 'warning' or $type eq 'error') {
+                    $do_dump = 1;
+                } else {
+                    $do_dump = 0;
+                }
+
+                $msgdata->{'dump_' . $type . '_messages'} = $do_dump;
+            }
+
+            if (my $code = $msgdata->{ $type . "_messages_callback"}) {
+                $code->($self,$msg);
+            } else {
+                # To support the deprecated non-command messaging API
+                my $deprecated_msgdata = UR::Object->_get_msgdata();
+                my $code = $deprecated_msgdata->{ $type . "_messages_callback"};
+                if ($code) {
+                    $code->($self,$msg) if ($code);
+                }
+            }
+
+            $msgdata->{ $type . "_message" } = $msg;
+
+            # If the callback set $msg to undef with "$_[1] = undef", then they didn't want the message
+            # processed further
+            return unless defined($msg);
+
+            if (my $fh = $msgdata->{ "dump_" . $type . "_messages" }) {
+                if ( $type eq 'usage' ) {
+                    (ref($fh) ? $fh : $stdout)->print((($type eq "status" or $type eq 'usage') ? () : (uc($type), ": ")), (defined($msg) ? $msg : ""), "\n");
+                }
+                else {
+                    (ref($fh) ? $fh : $stderr)->print((($type eq "status" or $type eq 'usage') ? () : (uc($type), ": ")), (defined($msg) ? $msg : ""), "\n");
+                }
+            }
+            if ($msgdata->{ "queue_" . $type . "_messages"}) {
+                my $a = $msgdata->{ $type . "_messages_arrayref" } ||= [];
+                push @$a, $msg;
+            }
+
+            my ($package, $file, $line, $subroutine) = caller;
+            $msgdata->{ $type . "_package" } = $package;
+            $msgdata->{ $type . "_file" } = $file;
+            $msgdata->{ $type . "_line" } = $line;
+            $msgdata->{ $type . "_subroutine" } = $subroutine;
+        }
+
+        return $msgdata->{ $type . "_message" };
+    };
+
+    my $messageinfo_subname = $type . "_message_source";
+    my $messageinfo_subref = sub {
+        my $self = shift;
+        my $msgdata = $self->_get_msgdata;
+        my %return;
+        my @keys = map { $type . $_ } qw( _message _package _file _line _subroutine );
+        @return{@keys} = @$msgdata{@keys};
+        return %return;
+    };
+
+    my $arrayref_subname = $type . "_messages_arrayref";
+    my $arrayref_subref = sub {
+        my $self = shift;
+        my $msgdata = $self->_get_msgdata;
+        $msgdata->{$type . "_messages_arrayref"} ||= [];
+        return $msgdata->{$type . "_messages_arrayref"};
+    };
+
+
+    my $array_subname = $type . "_messages";
+    my $array_subref = sub {
+        my $self = shift;
+
+        my $msgdata = $self->_get_msgdata;
+        return ref($msgdata->{$type . "_messages_arrayref"}) ?
+               @{ $msgdata->{$type . "_messages_arrayref"} } :
+               ();
+    };
+
+    no strict;
+    no warnings;
+
+    *$logger_subname      = $logger_subref;
+    *$messageinfo_subname = $messageinfo_subref;
+    *$arrayref_subname    = $arrayref_subref;
+    *$array_subname       = $array_subref;
 }
 
+
 sub _current_call_stack
 {
     my @stack = reverse split /\n/, Carp::longmess("\t");
@@ -662,77 +762,6 @@ sub _current_call_stack
     return \@stack;
 }
 
-# class that stores and manages messages
-package UR::ModuleBase::Message;
-
-use Scalar::Util qw(weaken);
-
-##- use UR::Util;
-UR::Util->generate_readonly_methods
-(
-    text         => undef,
-    level        => undef,
-    package_name => undef,
-    call_stack   => [],
-    time_stamp   => undef,
-    owner_class  => undef,
-    owner_id     => undef,
-    type         => undef,
-);
-
-sub create
-{
-    my $class = shift;
-    my $obj = {@_};
-    bless ($obj,$class);
-   weaken $obj->{'owner_id'} if (ref($obj->{'owner_id'}));
-
-    return $obj;
-}
-
-sub owner
-{
-    my $self = shift;
-    my ($owner_class,$owner_id) = ($self->owner_class, $self->owner_id);
-    if (not defined($owner_id))
-    {
-        return $owner_class;
-    }
-    elsif (ref($owner_id))
-    {
-        return $owner_id;
-    }
-    else
-    {
-        return $owner_class->get($owner_id);
-    }
-}
-
-sub string
-{
-    my $self = shift;
-    "$self->{time_stamp} $self->{type}: $self->{text}\n";
-}
-
-sub _stack_item_params
-{
-    my ($self, $stack_item) = @_;
-    my ($function, $parameters, @parameters);
-
-    return unless ($stack_item =~ s/\) called at [^\)]+ line [^\)]+\s*$/\)/);
-
-    if ($stack_item =~ /^\s*([^\(]*)(.*)$/)
-    {
-        $function = $1;
-        $parameters = $2;
-        @parameters = eval $parameters;
-        return ($function, @parameters);
-    }
-    else
-    {
-        return;
-    }
-}
 
 1;
 __END__
