@@ -5,7 +5,7 @@ use warnings;
 use File::Find;
 
 require UR;
-our $VERSION = "0.32"; # UR $VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::Namespace',
@@ -15,6 +15,12 @@ UR::Object::Type->define(
         domain                  => { is => 'Text', len => undef },
         allow_sloppy_primitives  => { is => 'Boolean', default_value => 1,
                                     doc => 'when true, unrecognized data types will function as UR::Value::SloppyPrimitive' },         
+        method_resolution_order => {
+            is => 'Text',
+            doc => 'Method Resolution Order to use for this namespace. C3 is only supported in Perl >= 5.9.5.',
+            value => ($^V lt v5.9.5 ? 'dfs' : 'c3'),
+            valid_values => ($^V lt v5.9.5 ? ['dfs'] : ['dfs', 'c3']),
+        },
     ],
 );
 
@@ -152,7 +158,7 @@ sub _get_class_names_under_namespace
         return if $File::Find::name =~ /\/\.deleted\//;   # .deleted directories are created by ur update classes
         return if -e $File::Find::name . '/UR_IGNORE';    # ignore a whole directory?
         return unless $File::Find::name =~ m/\.pm$/;      # must be a perl module
-        return unless $File::Find::name =~ m/($namespace\/.*)\.pm/;
+        return unless $File::Find::name =~ m/.*($namespace\/.*)\.pm/;
 
         my $try_class = $1;
         return if $try_class =~ m([^\w/]);  # Skip names that make for illegal package names.  Must be word chars or a /
@@ -161,10 +167,16 @@ sub _get_class_names_under_namespace
     };
 
     my @dirs_to_search = @INC;
+    my $path_to_check = $namespace;
+    $path_to_check .= "/$subdir" if $subdir;
+
+    @dirs_to_search = map($_ . '/' . $path_to_check, @dirs_to_search); # only look in places with namespace_name as a subdir
     unshift(@dirs_to_search, $namespace_dir) if (-d $namespace_dir);
-    @dirs_to_search = grep { $_ =~ m/\/$namespace/ or -d $_ . "/$namespace" }
-                      @dirs_to_search;  # only look in places with namespace_name as a subdir
-    find({ wanted => $wanted, preprocess => $preprocess },@dirs_to_search);
+
+    @dirs_to_search = grep { $_ =~ m/\/$path_to_check/ and -d $_ }
+                      @dirs_to_search;
+    return unless @dirs_to_search;
+    find({ wanted => $wanted, preprocess => $preprocess }, @dirs_to_search);
     return sort keys %class_names;
 }
 
