@@ -2,10 +2,10 @@ package UR::Namespace::Command::Define::Class;
 use strict;
 use warnings;
 use UR;
-our $VERSION = $UR::VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 class UR::Namespace::Command::Define::Class {
-    is => 'UR::Namespace::Command',
+    is => 'UR::Namespace::Command::Base',
     has => [
         names => {
             is_optional => 1,
@@ -45,8 +45,6 @@ sub execute {
         return;
     }
 
-    $self->_init or return;
-
     my $namespace = $self->namespace_name;
     my $is = $self->extends || 'UR::Object';
     my $parent_class_meta = UR::Object::Type->get($is);
