@@ -4,10 +4,11 @@ package UR::Namespace::Command::Old::DiffRewrite;
 use strict;
 use warnings;
 use UR;
+our $VERSION = "0.38"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
-    is => "UR::Namespace::Command",
+    is => "UR::Namespace::Command::Base",
 );
 
 sub help_description { 
