@@ -6,7 +6,7 @@ use above 'UR';
 
 package UR::Namespace::Command::Test::Window;
 require UR;
-our $VERSION = $UR::VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
@@ -35,12 +35,12 @@ sub execute {
         }
     }
     else {
-        DebugWindow->activate_with_gtk_support;
+        UR::Namespace::Command::Test::Window::Tk->activate_with_gtk_support;
     }
 
 }
 
-package DebugWindow;
+package UR::Namespace::Command::Test::Window::Tk;;
 
 our $tmp;
 our $workspace;
