@@ -2,6 +2,8 @@ package UR::Namespace::View::SchemaBrowser::CgiApp::Class;
 
 use strict;
 use warnings;
+require UR;
+our $VERSION = "0.38"; # UR $VERSION;
 
 use base 'UR::Namespace::View::SchemaBrowser::CgiApp::Base';
 
@@ -9,7 +11,7 @@ use Class::Inspector;
 
 sub setup {
 my($self) = @_;
-$DB::single=1;
+#$DB::single = 1;
     $self->start_mode('show_class_page');
     $self->mode_param('rm');
     $self->run_modes(
@@ -19,7 +21,7 @@ $DB::single=1;
 
 sub show_class_page {
 my $self = shift;
-$DB::single=1;
+#$DB::single = 1;
 
     my @namespace_names = $self->GetNamespaceNames();
     my $namespace_name = $self->namespace_name;
@@ -58,7 +60,7 @@ $DB::single=1;
     if ($class_obj) {
         my @class_detail;
         foreach my $prop_name ( qw( namespace doc er_role is_abstract is_final is_singleton
-                                    sub_classification_meta_class_name subclassify_by type_name) ) {
+                                    sub_classification_meta_class_name subclassify_by) ) {
 
             push @class_detail, { PROPERTY_NAME => $prop_name, PROPERTY_VALUE => $class_obj->$prop_name };
         }
