@@ -8,7 +8,7 @@ require Term::ANSIColor;
 use UR;
 use UR::Object::Command::List::Style;
 
-our $VERSION = "0.32"; # UR $VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 class UR::Object::Command::List {
     is => 'Command',
@@ -20,12 +20,13 @@ class UR::Object::Command::List {
         filter => {
             is => 'Text',  
             is_optional => 1,
-            doc => 'Filter results based on the parameters.  See below for how to.'
+            doc => 'Filter results based on the parameters.  See below for how to.',
+            shell_args_position => 1,
         },
         show => {
             is => 'Text',
             is_optional => 1,
-            doc => 'Specify which columns to show, in order.' 
+            doc => 'Specify which columns to show, in order.',
         },
         order_by => {
             is => 'Text',
@@ -60,10 +61,10 @@ class UR::Object::Command::List {
         _fields => {
             is_many => 1,
             is_optional => 1,
-            doc => 'Methods which the caller intends to use on the fetched objects.  May lead to pre-fetching the data.'
+            doc => 'Methods which the caller intends to use on the fetched objects.  May lead to pre-fetching the data.',
         },
     ], 
-    doc => 'lists objects matching specified params'
+    doc => 'lists objects matching specified params',
 };
 
 sub sub_command_sort_position { .2 };
@@ -106,14 +107,37 @@ sub create {
 sub _resolve_boolexpr {
     my $self = shift;
 
-    my ($bool_expr, %extra) = UR::BoolExpr->resolve_for_string(
-        $self->subject_class_name, 
-        $self->_complete_filter, 
-        $self->_hint_string,
-        $self->order_by,
-    );
+    my ($bool_expr,%extra);
+    eval {
+        ($bool_expr, %extra) = UR::BoolExpr->resolve_for_string(
+                                   $self->subject_class_name,
+                                   $self->_complete_filter,
+                                   $self->_hint_string,
+                                   $self->order_by,
+                               );
+    };
+    my $error = $@;
 
-    $self->error_message( sprintf('Unrecognized field(s): %s', join(', ', keys %extra)) )
+    unless ($bool_expr) {
+        eval {
+            ($bool_expr, %extra) = UR::BoolExpr->_old_resolve_for_string(
+                                   $self->subject_class_name,
+                                   $self->_complete_filter,
+                                   $self->_hint_string,
+                                   $self->order_by,
+                           )
+        };
+        if ($bool_expr) {
+            $self->warning_message("Failed to parse your query, but it was recognized by the deprecated filter parser.\n  Try putting quotes around the entire filter expression.\n  Use double quotes if your filter already includes single quotes, and vice-versa.\n  Values containing spaces need quotes around them as well\n  The error from the parser was:\n    $error");
+        } else {
+            die $error if $error;
+        }
+    }
+
+
+    #$self->error_message( sprintf('Unrecognized field(s): %s', join(', ', keys %extra)) )
+    $self->error_message( sprintf('Cannot list for class %s because some items in the filter or show were not properties of that class: %s',
+                          $self->subject_class_name, join(', ', keys %extra)))
         and return if %extra;
 
     return $bool_expr;
@@ -138,12 +162,11 @@ sub execute {
     my $bool_expr = $self->_resolve_boolexpr();
     return unless (defined $bool_expr);
   
-    # preloading the data ensures that the iterator doesn't trigger requery
-    my @results = $self->subject_class_name->get($bool_expr);
-
-    # TODO: remove the iterator entirely from the lister since all of the data is above--ss
+    # TODO: instead of using an iterator, get all the results back in a list and
+    # have the styler use the list, since it needs all the results to space the columns
+    # out properly anyway
     my $iterator;
-    unless ($iterator = $self->subject_class_name->create_iterator(where => $bool_expr)) {
+    unless ($iterator = $self->subject_class_name->create_iterator($bool_expr)) {
         $self->error_message($self->subject_class_name->error_message);
         return;
     }
@@ -231,10 +254,32 @@ EOS
         $doc .= " $help_synopsis\n";
     } else {
         $doc .= <<EOS
- lister-command --filter name=Bob --show id,name,address --order name
- lister-command --filter name='something with space',employees\>200,job~%manager
- lister-command --filter cost:20000-90000
- lister-command --filter answer:yes/maybe
+ list-cmd --filter name=Bob --show id,name,address --order name
+ list-cmd --filter name='something with space',employees\>200,job~%manager
+ list-cmd --filter cost:20000-90000
+ list-cmd --filter answer:yes/maybe
+
+Extended Syntax:
+----------------
+The filter expression may also use a more free-form syntax with arbitrary
+nesting of parentheses and 'and' or 'or' clauses.  This syntax accepts the
+words 'in', 'like' and 'between' in place of the above ':' and '~' operators.
+In addition, the in-list of values must begin with a left bracket, end with
+a right bracket and the values are separated with commas.
+
+This extended syntax expression will most likely contain spaces or other
+characters having special meaning to the shell, so they will need to be
+escaped with literal backslashes or enclosed in quotes of some kind.
+
+Extended Syntax Examples:
+-------------------------
+ list-cmd --filter 'name=Bob or address like "%main st"'
+ list-cmd --filter 'name="something with space" and (score < 10 or score > 100)'
+ list-cmd --filter 'cost between 20000-90000'
+ list-cmd --filter 'answer in [yes,maybe]'
+
+
+
 EOS
     }
 
@@ -278,8 +323,8 @@ EOS
                 my $data_type = $property->data_type || '';
                 $data_type = ucfirst(lc $data_type);
 
-                $doc .= sprintf(" %${longest_name}s  ($data_type): $property_doc\n",
-                                $property->property_name);
+                $doc .= sprintf(" %${longest_name}s  (%s): %s\n",
+                                $property->property_name,$data_type, $property_doc);
             }
         }
         else {
@@ -339,7 +384,9 @@ sub _subject_class_filterable_properties {
     my %props = map { $_->property_name => $_ }
                     $self->subject_class->property_metas;
 
-    return sort { $a->property_name cmp $b->property_name }
+    return map { $_->[1] }                   # These maps are to get around a bug in perl 5.8 sort
+           sort { $a->[0] cmp $b->[0] }      # involving method calls inside the sort sub that may
+           map { [ $_->property_name, $_ ] } # do sorts of their own
            grep { substr($_->property_name, 0, 1) ne '_' }  # Skip 'private' properties starting with '_'
            grep { ! $_->data_type or index($_->data_type, '::') == -1 }  # Can't filter object-type properties from a lister, right?
            values %props;
