@@ -1,11 +1,12 @@
 # Index for cached objects.
 
 package UR::Object::Index;
-our $VERSION = $UR::VERSION;;
+our $VERSION = "0.38"; # UR $VERSION;;
 use base qw(UR::Object);
 
 use strict;
 use warnings;
+require UR;
 
 
 # wrapper for one of the ID properties to make it less ugly
@@ -65,23 +66,17 @@ sub get_objects_matching
     my $value;
     for $value (@_)
     {               
-        if (not ref($value))
-        {
-            # property => value
-            @hr = grep { $_ } map { $_->{$value} } @hr;
-        }           
-        elsif (ref($value) eq "ARRAY") 
-        {
-            # property => [ v1, v2, v3]
-            @hr = grep { $_ } map { @$_{@$value} } @hr;
-        } 
-        elsif(ref($value) eq "HASH") 
+        my $value_ref = ref($value);
+        if($value_ref eq "HASH")
         {
             # property => { operator => "not like", value => "H~_WGS%", escape "~" }
             if (my $op = $value->{operator})
             {
                 $op = lc($op);
-                if ($op eq 'like' or $op eq 'not like')
+                if ($op eq '=') {
+                   @hr = grep { $_ } map { $_->{$value->{'value'}} } @hr;
+                }
+                elsif ($op eq 'like' or $op eq 'not like')
                 {
                     my $not = 1 if (substr($op,0,1) eq 'n');
                     my $comparison_value = $value->{value};                        
@@ -276,8 +271,18 @@ sub get_objects_matching
             {
                 Carp::confess("No operator specified in hashref value!" . Dumper($value));
             }
-        } 
-    }            
+        }
+        elsif (not $value_ref)
+        {
+            # property => value
+            @hr = grep { $_ } map { $_->{$value} } @hr;
+        }
+        elsif ($value_ref eq "ARRAY")
+        {
+            # property => [ v1, v2, v3]
+            @hr = grep { $_ } map { @$_{@$value} } @hr;
+        }
+    }
     return (map { values(%$_) } @hr);
 }
 
@@ -307,7 +312,7 @@ sub _build_data_tree
     
     # _add_object in bulk.
     my ($object,@values,$hr,$value);
-    for my $object ($UR::Context::current->all_objects_loaded($self->indexed_class_name)) {            
+    for my $object ($UR::Context::current->all_objects_loaded($indexed_class_name)) {
         if (@indexed_property_names) {
             @values = map { my $val = $object->$_; defined $val ? $val : undef } @indexed_property_names;
             @values = (undef) unless(@values);
@@ -318,10 +323,13 @@ sub _build_data_tree
             no warnings 'uninitialized';  # in case $value is undef
             $hr->{$value} ||= {};
             $hr = $hr->{$value};
-        }            
-        $hr->{$object->id} = $object;
+        }
+        my $obj_id = $object->id;
+        $hr->{$obj_id} = $object;
+        if (Scalar::Util::isweak($UR::Context::all_objects_loaded->{$indexed_class_name}->{$obj_id})) {
+            Scalar::Util::weaken($hr->{$obj_id});
+        }
     }
-    
 }
 
 # FIXME maybe objects in an index should always be weakend?
