@@ -2,6 +2,8 @@ package UR::Object::Property::View::Default::Text;
 
 use strict;
 use warnings;
+require UR;
+our $VERSION = "0.38"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
