@@ -3,7 +3,7 @@ use warnings;
 use strict;
 
 require UR;
-our $VERSION = "0.32"; # UR $VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 use Sys::Hostname;
 use Cwd;
@@ -68,12 +68,12 @@ sub property_meta_for_name {
             push @pmeta, $property_meta;
             last if $link eq $chain[-1];
             my @joins = UR::Object::Join->resolve_chain($last_class_name, $link);
-            unless (@joins) {
-                Carp::confess("No joins for $full_link?");
-            }
+            return unless @joins;
+
             $last_class_name = $joins[-1]{foreign_class};
             $last_class_meta = $last_class_name->__meta__;
         }
+        return unless (@pmeta and $pmeta[-1]);
         return @pmeta if wantarray;
         return $pmeta[-1];
     }
@@ -99,6 +99,30 @@ sub property_meta_for_name {
     return;
 }
 
+# A front-end for property_meta_for_name, but
+# will translate the generic 'id' property into the class' real ID property,
+# if it's not called 'id'
+sub _concrete_property_meta_for_class_and_name {
+    my($self,$property_name) = @_;
+
+    my $property_meta = $self->property_meta_for_name($property_name);
+
+    if ($property_meta
+        and $property_meta->class_name eq 'UR::Object'
+        and $property_meta->property_name eq 'id')
+    {
+        # This is the generic id property.  Remap it to the class' real ID property name
+        my @id_properties = $self->id_property_names;
+        if (@id_properties == 1 and $id_properties[0] eq 'id') {
+            return $property_meta;
+        }
+        return map { $self->_concrete_property_meta_for_class_and_name($_) } @id_properties;
+    }
+    return $property_meta;
+}
+
+
+
 sub _flatten_property_name {
     my ($self, $name) = @_;
     
@@ -146,7 +170,8 @@ sub direct_id_property_metas {
         map { $DIRECT_ID_PROPERTY_METAS_TEMPLATE->get_rule_for_values($class_name, $_, 0) }
         @{$self->{'id_by'}};
 
-    @id_property_objects = sort { $a->is_id <=> $b->is_id } @id_property_objects;
+    my $sort_sub = sub ($$) { return $_[0]->is_id cmp $_[1]->is_id };
+    @id_property_objects = sort $sort_sub @id_property_objects;
     if (@id_property_objects == 0) {
         @id_property_objects = $self->property_meta_for_name("id");
     }
@@ -158,28 +183,30 @@ sub parent_class_names {
     return @{ $self->{is} };
 }
 
+push @cache_keys, '_id_property_names';
 sub id_property_names {
     # FIXME Take a look at id_property_names and all_id_property_names.  
     # They look extremely similar, but tests start dying if you replace one
     # with the other, or remove both and rely on the property's accessor method
 
     my $self = _object(shift);
-    my @id_by;
-    #my $template = UR::BoolExpr::Template->resolve('UR::Object::Type', 'class_name');
 
-    unless ($self->{id_by} and @id_by = @{ $self->{id_by} }) {
-        foreach my $parent ( @{ $self->{'is'} } ) {
-            my $parent_class = UR::Object::Type->get(class_name => $parent);
-            #my $rule = $template->get_rule_for_values($parent);
-            #my $parent_class = $UR::Context::current->get_objects_for_class_and_rule('UR::Object::Type', $rule);
-            next unless $parent_class;
-            @id_by = $parent_class->id_property_names;
-            last if @id_by;
+    unless ($self->{'_id_property_names'}) {
+        my @id_by;
+        unless ($self->{id_by} and @id_by = @{ $self->{id_by} }) {
+            foreach my $parent ( @{ $self->{'is'} } ) {
+                my $parent_class = $parent->class->__meta__;
+                next unless $parent_class;
+                @id_by = $parent_class->id_property_names;
+                last if @id_by;
+            }
         }
-    }   
-    return @id_by;    
+        $self->{'_id_property_names'} = \@id_by;
+    }
+    return @{$self->{'_id_property_names'}};
 }
 
+push @cache_keys, '_all_id_property_names';
 sub all_id_property_names {
     # return shift->id_property_names(@_); This makes URT/t/99_transaction.t fail
     my $self = shift;
@@ -509,7 +536,15 @@ sub sorter {
     my $sorter = $self->{_sorter}{$key};
     unless ($sorter) {
         my @is_numeric;
+        my @is_descending;
         for my $property (@properties) {
+            if ($property =~ m/^(-|\+)(.*)$/) {
+                push @is_descending, $1 eq '-';
+                $property = $2;  # yes, we're manipulating the original list element
+            } else {
+                push @is_descending, 0;
+            }
+
             my $pmeta;
             if ($self->isa("UR::Object::Set::Type")) {
                 # If we're a set, we want to examine the property of our members.
@@ -531,17 +566,19 @@ sub sorter {
                 push @is_numeric, 0;
             }
         }
+
         no warnings;   # don't print a warning about undef values ...alow them to be treated as 0 or '' 
         $sorter = $self->{_sorter}{$key} ||= sub($$) {
             for (my $n = 0; $n < @properties; $n++) {
                 my $property = $properties[$n];
-                my $cmp;
-                if ($is_numeric[$n]) {
-                    $cmp = ($_[0]->$property <=> $_[1]->$property);
+                my($first,$second) = $is_descending[$n] ? ($_[1]->$property,$_[0]->$property) : ($_[0]->$property,$_[1]->$property);
+                if (!defined($second)) {
+                    return -1;
+                } elsif (!defined($first)) {
+                    return 1;
                 }
-                else {
-                    $cmp = ($_[0]->$property cmp $_[1]->$property);
-                }                    
+
+                my $cmp = $is_numeric[$n] ? $first <=> $second : $first cmp $second;
                 return $cmp if $cmp;
             }
             return 0;
@@ -634,33 +671,39 @@ sub autogenerate_new_object_id_datasource {
 
 # Support the autogeneration of unique IDs for objects which require them.
 sub autogenerate_new_object_id {
-    my $self = shift;
-    my $rule = shift;
+    my $self = _object($_[0]);
+    #my $rule = shift;
 
-    my $id_generator = $self->id_generator;
+    unless ($self->{'_resolved_id_generator'}) {
+        my $id_generator = $self->id_generator;
 
-    if (ref($id_generator) eq 'CODE') {
-        return $id_generator->($self,$rule);
+        if (ref($id_generator) eq 'CODE') {
+            $self->{'_resolved_id_generator'} = $id_generator;
 
-    } elsif ($id_generator and $id_generator =~ m/^\-(\S+)/) {
-        my $id_method = 'autogenerate_new_object_id_' . $1;
-        unless ($self->can($id_method)) {
-            Carp::croak("'$id_generator' is an invalid id_generator for class "
-                        . $self->class_name
-                        . ": Can't locate object method '$id_method' via package ".ref($self));
-        }
-        return $self->$id_method($rule);
+        } elsif ($id_generator and $id_generator =~ m/^\-(\S+)/) {
+            my $id_method = 'autogenerate_new_object_id_' . $1;
+            my $subref = $self->can($id_method);
+            unless ($subref) {
+                Carp::croak("'$id_generator' is an invalid id_generator for class "
+                            . $self->class_name
+                            . ": Can't locate object method '$id_method' via package ".ref($self));
+            }
+            $self->{'_resolved_id_generator'} = $subref;
 
-    } else {
-        # delegate to the data source
-        my ($data_source) = $UR::Context::current->resolve_data_sources_for_class_meta_and_rule($self);
-        if ($data_source) {
-            return $data_source->autogenerate_new_object_id_for_class_name_and_rule(
-                $self->class_name,
-                $rule
-            )
+        } else {
+            # delegate to the data source
+            my ($data_source) = $UR::Context::current->resolve_data_sources_for_class_meta_and_rule($self);
+            if ($data_source) {
+                $self->{'_resolved_id_generator'} = sub {
+                    $data_source->autogenerate_new_object_id_for_class_name_and_rule(
+                        shift->class_name,
+                        shift
+                    )
+                };
+            }
         }
     }
+    goto $self->{'_resolved_id_generator'};
 }
 
 # from ::Object->generate_support_class
@@ -746,7 +789,6 @@ sub generate_support_class_for_extension {
         my $class_props = UR::Util::deep_copy($subject_class_obj->{has});    
         for (values %$class_props) {
             delete $_->{class_name};
-            delete $_->{type_name};
             delete $_->{property_name};
         }
         
@@ -755,7 +797,6 @@ sub generate_support_class_for_extension {
                 class_name => $new_class_name,
                 is => \@parent_class_names, 
                 is_abstract => 0,
-                type_name => $subject_class_obj->type_name . " ghost",
                 has => [%$class_props],
                 attributes_have => $attributes_have,
                 id_properties => \@id_property_names,
@@ -1163,7 +1204,7 @@ sub table_for_property {
         my $property_object = UR::Object::Property->get( class_name => $class_object->class_name, property_name => $property_name );
         if ( $property_object )
         {
-            return unless $property_object->column_name;
+            next unless $property_object->column_name;
             return $class_object->table_name;
         }
     }
@@ -1209,7 +1250,6 @@ sub property_for_column {
     return;
 }
 
-
 # Methods for maintaining unique constraints
 # This is primarily used by the class re-writer (ur update classes-from-db), but
 # BoolExprs use them,too
@@ -1332,7 +1372,7 @@ sub _property_change_callback {
 
     if ($method eq 'create') {
         unless ($class_obj->{'has'}->{$property_name}) {
-            my @attr = qw( class_name attribute_name data_length data_type is_delegated is_optional property_name type_name );
+            my @attr = qw( class_name data_length data_type is_delegated is_optional property_name );
 
             my %new_property;
             foreach my $attr_name (@attr ) {
@@ -1461,6 +1501,44 @@ sub __signal_change__ {
     return @rv;
 }
 
+our %STANDARD_VALID_SIGNALS = ( create        => 1,
+                                'delete'      => 1,
+                                commit        => 1,
+                                rollback      => 1,
+                                load          => 1,
+                                unload        => 1,
+                                load_external => 1 );
+sub _is_valid_signal {
+    my $self = shift;
+    my $aspect = shift;
+
+    # Undefined attributes indicate that the subscriber wants any changes at all to generate a callback.
+    return 1 if (! defined $aspect);
+
+    # All standard creation and destruction methods emit a signal.
+    return 1 if ($STANDARD_VALID_SIGNALS{$aspect});
+
+    for my $property ($self->all_property_names)
+    {
+        return 1 if $property eq $aspect;
+    }
+
+    if (!exists $self->{'_is_valid_signal'}) {
+        $self->{'_is_valid_signal'} = { map { $_ => 1 } @{$self->{'valid_signals'}} };
+    }
+
+    return 1 if ($self->{'_is_valid_signal'}->{$aspect});
+
+    foreach my $parent_meta ( $self->parent_class_metas ) {
+        if ($parent_meta->_is_valid_signal($aspect)) {
+            $self->{'_is_valid_signal'}->{$aspect} = 1;
+            return 1;
+        }
+    }
+
+    return 0;
+}
+
 
 sub generated {
     my $self = shift;
