@@ -3,13 +3,13 @@ package UR::Object::View::Default::Gtk2;
 use strict;
 use warnings;
 require UR;
-our $VERSION = "0.32"; # UR $VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 class UR::Object::View::Default::Gtk2 {
     is => 'UR::Object::View',
     has_constant => [
-        perspective => 'default',
-        toolkit => 'gtk2'
+        perspective => { value => 'default'},
+        toolkit => { value => 'gtk2'},
     ],
 };
 
@@ -33,7 +33,11 @@ sub _update_view_from_subject {
         $text .= " (REUSED ADDR)\n";
     } else {
         $text .= "\n";
-        for my $aspect (sort { $a->position <=> $b->position } @aspects) {       
+        my @sorted_aspects = map { $_->[1] }
+                             sort { $a->[0] <=> $b->[0] }
+                             map { [ $_->position, $_ ] }
+                             @aspects;
+        for my $aspect (@sorted_aspects) {
             my $label = $aspect->label;
             $text .= "\n" . $label . ": ";
             if ($subject) {
