@@ -2,13 +2,16 @@ package UR::Object::View::Default::Html;
 
 use strict;
 use warnings;
+require UR;
+our $VERSION = "0.38"; # UR $VERSION;
 use IO::File;
 
 class UR::Object::View::Default::Html {
     is => 'UR::Object::View::Default::Xsl',
     has => {
         output_format => { value => 'html' },
-        transform => { value => 1 }
+        transform => { value => 1 },
+        toolkit => { value => 'html' },
     }
 };
 
