@@ -2,6 +2,8 @@ package UR::Object::View::Default::Json;
 
 use strict;
 use warnings;
+require UR;
+our $VERSION = "0.38"; # UR $VERSION;
 
 use JSON;
 
@@ -10,7 +12,8 @@ class UR::Object::View::Default::Json {
     has_constant => [
         toolkit     => { value => 'json' },
     ],
-    has => [
+    has_optional => [
+        encode_options => { is => 'ARRAY', default_value => ['ascii', 'pretty', 'allow_nonref', 'canonical'], doc => 'Options to enable on the JSON object; see the documentation for the JSON Perl module' },
     ],
 };
 
@@ -18,12 +21,22 @@ my $json;
 sub _json {
     my ($self) = @_;
     return $json if defined $json;
-    return $json = JSON->new->ascii->pretty->allow_nonref;
+
+    $json = JSON->new;
+    foreach my $opt ( @{ $self->encode_options } ) {
+        eval { $json = $json->$opt; };
+        if ($@) {
+            Carp::croak("Can't initialize JSON object for encoding.  Calling method $opt from encode_options died: $@");
+        }
+        if (!$json) {
+            Carp::croak("Can't initialize JSON object for encoding.  Calling method $opt from encode_options returned false");
+        }
+    }
+    return $json;
 }
 
 sub _generate_content {
     my $self = shift;
-
     return $self->_json->encode($self->_jsobj);
 }
 
@@ -34,15 +47,11 @@ sub _jsobj {
     return '' unless $subject;
 
     my %jsobj = ();
-    
-#    unless ($self->_subject_is_used_in_an_encompassing_view()) {
-        # the content for any given aspect is handled separately
-        for my $aspect ($self->aspects) { 
-                
-            my $val = $self->_generate_content_for_aspect($aspect);
-            $jsobj{$aspect->name} = $val if defined $val;
-        }
-#    }
+
+    for my $aspect ($self->aspects) { 
+        my $val = $self->_generate_content_for_aspect($aspect);
+        $jsobj{$aspect->name} = $val if defined $val;
+    }
 
     return \%jsobj;
 }
@@ -55,7 +64,7 @@ sub _generate_content_for_aspect {
     my $aspect_name = $aspect->name;
 
     my $aspect_meta = $self->subject_class_name->__meta__->property($aspect_name);
-    warn $aspect_name if ref($subject) =~ /Set/;
+    #warn $aspect_name if ref($subject) =~ /Set/;
 
     my @value;
     eval {
@@ -65,39 +74,31 @@ sub _generate_content_for_aspect {
         warn $@;
         return;
     }
-    
-    if (@value == 0) {
-        return; 
-    }
-        
-    if (Scalar::Util::blessed($value[0])) {
-        unless ($aspect->delegate_view) {
-            eval {
-                $aspect->generate_delegate_view;
-            };
-            if ($@) {
-                warn $@;
-            }
-        }
+
+    # Always look for a delegate view.
+    # This means we replace the value(s) with their
+    # subordinate widget content.
+    unless ($aspect->delegate_view) {
+        $aspect->generate_delegate_view;
     }
 
     my $ref = [];
- 
-    # Delegate to a subordinate view if needed.
-    # This means we replace the value(s) with their
-    # subordinate widget content.
+
     if (my $delegate_view = $aspect->delegate_view) {
         foreach my $value ( @value ) {
-            $delegate_view->subject($value);
+            if (Scalar::Util::blessed($value)) {
+                $delegate_view->subject($value);
+            } else {
+                $delegate_view->subject_id($value);
+            }
             $delegate_view->_update_view_from_subject();
-            
+
             if ($delegate_view->can('_jsobj')) {
                 push @$ref, $delegate_view->_jsobj;
             } else {
                 my $delegate_text = $delegate_view->content();
-
                 push @$ref, $delegate_text;
-            }            
+            }
         }
     }
     else {
@@ -110,7 +111,7 @@ sub _generate_content_for_aspect {
         }
     }
 
-    if ($aspect_meta->is_many) {
+    if ($aspect_meta && $aspect_meta->is_many) {
         return $ref;
     } else {
         return shift @$ref;
