@@ -4,13 +4,17 @@ use strict;
 use warnings;
 
 require UR;
-our $VERSION = "0.32"; # UR $VERSION;
+our $VERSION = "0.38"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => 'UR::Value::FilePath',
     is => ['UR::Value::FilesystemPath'],
-    english_name => 'file path',
 );
 
+sub line_count {
+    my $self = shift;
+    my ($line_count) = qx(wc -l $self) =~ /^\s*(\d+)/;
+    return $line_count;
+}
+
 1;
-#$Header$
